/* Main code for TimeShare */

#include <Pilot.h>
#include "callback.h"

#include "timeshareRsc.h"
#include <System/ExgMgr.h>

void BeamSend(void);

static Boolean MainFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    
    CALLBACK_PROLOGUE

    switch (e->eType) {
    case frmOpenEvent:
	frm = FrmGetActiveForm();
	FrmDrawForm(frm);
	handled = true;
	break;

    case menuEvent:
	MenuEraseStatus(NULL);

	switch(e->data.menu.itemID) {
	}

    	handled = true;
	break;

    case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
        case 1:
            BeamSend();
            handled = true;
            break;
	}
	break;

    case keyDownEvent:
    switch(e->data.keyDown.chr) {
        case sendDataChr:
            BeamSend();
            handled = true;
            break;
    }
    break;

    default:
        break;
    }

    CALLBACK_EPILOGUE

    return handled;
}

static Boolean ApplicationHandleEvent(EventPtr e)
{
    FormPtr frm;
    Word    formId;
    Boolean handled = false;

    if (e->eType == frmLoadEvent) {
	formId = e->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	switch(formId) {
	case MainForm:
	    FrmSetEventHandler(frm, MainFormHandleEvent);
	    break;
	}
	handled = true;
    }

    return handled;
}

/* Get preferences, open (or create) app database */
static Word StartApplication(void)
{
    FrmGotoForm(MainForm);
    return 0;
}

/* Save preferences, close forms, close app database */
static void StopApplication(void)
{
    FrmSaveAllForms();
    FrmCloseAllForms();
}

/* The main event loop */
static void EventLoop(void)
{
    Word err;
    EventType e;

    do {
	EvtGetEvent(&e, evtWaitForever);
	if (! SysHandleEvent (&e))
	    if (! MenuHandleEvent (NULL, &e, &err))
		if (! ApplicationHandleEvent (&e))
		    FrmDispatchEvent (&e);
    } while (e.eType != appStopEvent);
}

void BeamSend(void) {
    ExgSocketType exgSock;
    Err err;
    ULong secs;
    DWord romversion;
    UInt bytes_sent;

    /* Check for IR beaming capability */
    FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion);
    if (romversion < 0x03003000) {
        FrmAlert(NoBeamForm);
        return;
    }

    /* Zero the exchange socket */
    MemSet(&exgSock, sizeof(ExgSocketType), 0);

    /* Set the name and description */
    exgSock.name = MemPtrNew(13);
    StrCopy(exgSock.name, "Correct Time");
    exgSock.description = MemPtrNew(13);
    StrCopy(exgSock.description, "Correct Time");

    /* Set the target */
    exgSock.target = 'TmSh';
    exgSock.length = 4;
    //exgSock.count = 1;


    /* Do the beaming */
    err = ExgPut(&exgSock);
    secs = TimGetSeconds();
    if (!err) {
         while (1) {
            bytes_sent = ExgSend(&exgSock, &secs, 4, &err);
            if (bytes_sent < 4) FrmAlert(NoBeamForm);
            break;
         }
         ExgDisconnect(&exgSock, err);
    }

    /* Cleanup */
    MemPtrFree(exgSock.name);
    MemPtrFree(exgSock.description);

}

void BeamReceive(ExgSocketPtr sockP, Boolean appIsActive) {
    Err err;
    ULong remote_secs;
    UInt bytes_got;

    err = ExgAccept(sockP);
    if (!err) {
        while (1) {
            bytes_got = ExgReceive(sockP, &remote_secs, 4, &err);
            if (bytes_got == 4) TimSetSeconds(remote_secs);
            if (!bytes_got) break;
        }
        ExgDisconnect(sockP, err);
    }
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    Word err;
    Boolean appIsActive;

    if (cmd == sysAppLaunchCmdNormalLaunch) {

	err = StartApplication();
	if (err) return err;

	EventLoop();
	StopApplication();

    } else if (cmd == sysAppLaunchCmdExgReceiveData) {

        if (launchFlags&sysAppLaunchFlagSubCall) appIsActive = true;
        else appIsActive = false;
        BeamReceive((ExgSocketPtr)cmdPBP, appIsActive);

    }

    return 0;
}
